<?php
// +-----------------------------------------------------------------------+
// | Mod Name : secureImages                                               |
// | Mod Version : 0.5                                                     |
// | Mod Orignal author: acp <iacp@free.fr>                                |
// | Mod description : see install files                                   |
// |                                                                       |
// +-----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or modify  |
// | it under the terms of the GNU General Public License as published by  |
// | the Free Software Foundation                                          |
// |                                                                       |
// | This program is distributed in the hope that it will be useful, but   |
// | WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      |
// | General Public License for more details.                              |
// |                                                                       |
// | You should have received a copy of the GNU General Public License     |
// | along with this program; if not, write to the Free Software           |
// | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, |
// | USA.                                                                  |
// +-----------------------------------------------------------------------+

/**
 * SecImgCfg class
 *
 * It is responsible of handling all configuration options of the plugin.
 * In other words, it stores/retrieves options chosen by the user and
 * provides them to requesting methods.
 *
 * Storing/Retrieval code is inspired from PHPWG's wiki examples
 */
class SecImgCfg {

  var $cfg_opts = array();
  var $default_values = array(
      'htaccess_type'          => 'selective',  // .htaccess creation type.
      'exclude_extensions'     => array(),      // Extensions of files that should not be handled.
      'htaccess_extra_content' => '',           // Extra content added to created .htaccess files
      'allow_rep_thumbs'       => FALSE,        // Do not block access to thumbnails that reprensent public categories
      'max_file_size_value'    => 0,            // Do not handle files bigger than a certain value
      'max_file_size_unit'     => 'KB',          // Unit of the maximum handled file size ('KB'ytes, 'MB'ytes)
    );
  /**
   * Constructor - Loads configuration options or sets default values
   */
  function SecImgCfg() {

    $cfg_raw_data = @file_get_contents( dirname(__FILE__).'/../data.dat' );

    // Reading existing store configuratin options
    if ($cfg_raw_data !== FALSE) {
      $this->cfg_opts = unserialize($cfg_raw_data);
    }

    /**
     * Setting default values where needed. Usefull in two cases :
     * 1) No configuration file
     * 2) The configuration file is old, one new entry was added and was
     *    not initialized above
     */
    foreach ($this->default_values as $key => $value)
      if (!isset($this->cfg_opts[$key]))
        $this->cfg_opts[$key] = $value;
  }

  /**
   * store_config - Stores configuration options
   */
  function store_config() {
    $file = @fopen( dirname(__FILE__).'/../data.dat', 'w' );
    
    if ($file !== FALSE) {
      fwrite($file, serialize($this->cfg_opts) );
      fclose( $file );
      return TRUE;
    }
    else {
      return FALSE;
    }
  }

  /**
   * set - Set a configuration option
   *
   * @field: which option to set
   * @value: value to assign
   */
  function set($field, &$value) {
    if (!isset($this->cfg_opts[$field]))
      return; // Print error ? Maybe if we later have a log, add an entry

    $this->cfg_opts[$field] = $value;
  }

  /**
   * get - Get a REFERENCE to a configuration option
   *
   * @field: which option to get
   */
  function &get($field) {
    if (!isset($this->cfg_opts[$field]))
      return NULL; // Print error ? Maybe if we later have a log, add an entry

    return $this->cfg_opts[$field];
  }
  

} // End of SecImgCfg definition

/**
 * We instantiate an object here so as to be certain that if two
 * files want to use the class within the same PHP session, they will
 * both have the updated versions of the configuration options, which
 * are not necessarily the ones contained in the data.dat file
 */
$cfg_obj = new SecImgCfg;

?>
