<?php
// +-----------------------------------------------------------------------+
// | Mod Name : secureImages                                               |
// | Mod Version : 0.5                                                     |
// | Mod Orignal author: acp <iacp@free.fr>                                |
// | Mod description : see install files                                   |
// |                                                                       |
// +-----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or modify  |
// | it under the terms of the GNU General Public License as published by  |
// | the Free Software Foundation                                          |
// |                                                                       |
// | This program is distributed in the hope that it will be useful, but   |
// | WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      |
// | General Public License for more details.                              |
// |                                                                       |
// | You should have received a copy of the GNU General Public License     |
// | along with this program; if not, write to the Free Software           |
// | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, |
// | USA.                                                                  |
// +-----------------------------------------------------------------------+



$secureImages_error_labels = array(
  'SECUREIMAGES-HTACCESS-NOT-WRITEABLE' => array(
    'The file (or directory) is not writable',
    'You need to give permission to the script to write to all your galleries subdirectories, or create the .htaccess files and give write access only to them.'
    ),
  'SECUREIMAGES-HTACCESS-NOT-REMOVABLE' => array(
    'The file (or directory) can not be removed',
    'You do not seem to have the necessary rights to remove this file.'
    ),
  'SECUREIMAGES-UNKNOWN' => array(
    'Unkown error',
    'Something went wrong. If you cannot find a solution, post a message to phpwebgallery forums for guidance. You might have found a bug.'
    )
);

class HtaccesManagement {
  var $list; // Array storing all informations for each unhandled file
  var $storage_category_id; // Id of the storage category, whose path is in $path
  var $path; // Path in which we are working, whose id is in $storage_category_id
  var $simulate; // Simulation mode on ?
  var $return;  // Variable used to return infos/errors to administration panel

  /**
   * Constructor
   */
  function HtaccesManagement($storage_category_id, $path, $simulate, &$return) {
    $this->list = array();
    $this->storage_category_id = $storage_category_id;
    $this->path = $path;
    $this->simulate = $simulate;
    $this->return =& $return;
  }

  /**
   * add - append a new entry in the list of unhandled files
   *
   * @id: id of the file within the database
   * @filename: name of the file
   * @default_handling: should the file (and thumbnail/hd version)
   *                    be handled by the plugin ?
   */
  function add($id, &$filename, $default_handling = FALSE) {
    $new_elt = array( 'filename' => $filename,
                      'handle_thumb' => $default_handling,
                      'handle_normal' => $default_handling,
                      'handle_high' => $default_handling
                    );

    $this->list[$id] = $new_elt;
  }

  /**
   * changeHandleState - change the handling of a file
   *
   * @id: id of the file within the database
   * @type_of_file: enumerator indicating which "version"
   *                of the file we are dealing with
   *                . T -> thumbmail
   *                . N -> normal
   *                . H -> high
   * @value: boolean to set the handling state to 
   */
  function changeHandleState($id, $type_of_file, $value) {
    if (!isset($this->list[$id]))
      return;
    switch ($type_of_file) {
      case 'T': // Thumbnail
        $key = 'handle_thumb';
        break;
      case 'N': // Normal
        $key = 'handle_normal';
        break;
      case 'H': // High
        $key = 'handle_high';
        break;
      default:
        die('Unexpected type "'.$type_of_file.'" (Thumbnail, Normal, High) given to changeHandleState. Not normal, quitting.');
    }
    $this->list[$id][$key] = (boolean) $value;
  }

  /**
   * write - write the .htaccess file(s)
   */
  function write() {
    $this->_writeHtaccessFile('N');
    $this->_writeHtaccessFile('T');
    if (file_exists($this->path.'/pwg_high'))
      $this->_writeHtaccessFile('H');
  }

  /**
   * writeRootHtaccess - the galleries root is not a category, we need a special
   *                     function to write the .htaccess it contains
   */
  function writeRootHtaccess() {
    global $cfg_obj;

    $file = PHPWG_ROOT_PATH . '/galleries/.htaccess';

    // Can we write the .htaccess file ?
    if (!is_writable_extended($file)) {
      $this->logError($file, 'SECUREIMAGES-HTACCESS-NOT-WRITEABLE');
      return;
    }
    // If simulation mode, and file is writable, exit now
    elseif ($this->simulate) {
      $this->logInfo($file, 'successfully created');
      return;
    }

    $hdl = @fopen( $file, 'w' );
    
    if (!$hdl) {
      $this->logError($file, 'SECUREIMAGES-UNKNOWN');
      return;
    }

    // Did the user specify extra content for the generated .htaccess files ?
    $htaccess_extra_content = explode('__WRITE_HERE__', $cfg_obj->get('htaccess_extra_content'));
    if (count($htaccess_extra_content) > 1) {
      fwrite($hdl, current($htaccess_extra_content) . "\n");
      next($htaccess_extra_content);
    }
    fwrite($hdl, "deny from all\n");
    fwrite($hdl, current($htaccess_extra_content));
    fclose($hdl);
  }

  /**
   * _writeHtaccessFile - write a single .htacces file
   *
   * @type_of_file: enumerator of the "version" of files we
   *                are dealing with
   */
  function _writeHtaccessFile($type_of_file = 'N') {
    global $conf;
    global $cfg_obj;

    switch ($type_of_file) {
      case 'T': // Thumbnail
        $file = $this->path . '/thumbnail/.htaccess';
        $type_of_file_key = 'handle_thumb';
        break;
      case 'N': // Normal
        $file = $this->path . '/.htaccess';
        $type_of_file_key = 'handle_normal';
        break;
      case 'H': // High
        $file = $this->path . '/pwg_high/.htaccess';
        $type_of_file_key = 'handle_high';
        break;
      default:
        die('Unexpected type "'.$type_of_file.'" (Thumbnail, Normal, High) given to _writeHtaccessFile. Not normal, quitting.');
    }

    // Can we write the .htaccess file ?
    if (!is_writable_extended($file)) {
      $this->logError($file, 'SECUREIMAGES-HTACCESS-NOT-WRITEABLE');
      return;
    }
    // If simulation mode, and file is writable, exit now
    elseif ($this->simulate) {
      $this->logInfo($file, 'successfully created');
      return;
    }

    $hdl = @fopen( $file, 'w' );
    
    if (!$hdl) {
      $this->logError($file, 'SECUREIMAGES-UNKNOWN');
      return;
    }

    // Did the user specify extra content for the generated .htaccess files ?
    $htaccess_extra_content = explode('__WRITE_HERE__', $cfg_obj->get('htaccess_extra_content'));
    if (count($htaccess_extra_content) > 1) {
      fwrite($hdl, current($htaccess_extra_content) . "\n");
      next($htaccess_extra_content);
    }

    if ($_POST['htaccess_type'] == 'selective') {
      // Not handling file types selected in the exception settings of the administration panel
      if (isset($_POST['file_types']) && count($_POST['file_types']) > 0) {
        fwrite($hdl, '<FilesMatch "\.(' . 
          implode('|', $_POST['file_types']) . 
          ')$>' .
          "\nallow from all\n</FilesMatch>\n\n"
          );
      }

      // Not handling private thumbnails of public categories if requested
      if ( $type_of_file == 'T' && $cfg_obj->get('allow_rep_thumbs') ) {
        $query = '
          SELECT images.id AS img_id, images.path AS path
            FROM '.IMAGES_TABLE.' AS images 
              INNER JOIN '.CATEGORIES_TABLE.' AS categories
                ON images.id = categories.representative_picture_id 
            WHERE categories.status = "public" 
              AND images.storage_category_id = "' . $this->storage_category_id . '";';
        $result = pwg_query($query);

        while ($row = mysql_fetch_array($result)) {
          // If this file does not appear in our list, we create a new entry
          if (!isset($this->list[$row['img_id']])) {
            $temp_file = basename($row['path']);
            $this->add($row['img_id'], $temp_file, TRUE);
          }
          // Marking thumbnail as unhandled, which means that we allow direct access to it
          $this->changeHandleState($row['img_id'], 'T', FALSE);
        }
      }

      // Not handling files bigger than the limit set by the user (only possible
      // with normal and high files, when metadata synchronization was made)
      if ( $type_of_file != 'T' && $cfg_obj->get('max_file_size_value') > 0) {
        $filesize_field = ($type_of_file == 'H' ? 'high_' : '') .'filesize';
        $max_file_size = $cfg_obj->get('max_file_size_value') * ($cfg_obj->get('max_file_size_unit') == 'KB' ? 1 : 1024);

        $query = '
          SELECT id as img_id, path, ' . $filesize_field . ' AS filesize
            FROM '.IMAGES_TABLE.' AS images 
            WHERE storage_category_id = "' . $this->storage_category_id . '"' .
              ($type_of_file == 'H' ? 'AND has_high = "true"' : '') . '
              AND ' . $filesize_field . ' > ' . $max_file_size . ';';
        $result = pwg_query($query);

        while ($row = mysql_fetch_array($result)) {
          // If this file does not appear in our list, we create a new entry
          if (!isset($this->list[$row['img_id']])) {
            $temp_file = basename($row['path']);
            $this->add($row['img_id'], $temp_file, TRUE);
          }
          // Marking file as unhandled, which means that we allow direct access to it
          $this->changeHandleState($row['img_id'], $type_of_file, FALSE);
        }
      }

      // Creating list of files that will not be handled, in order to allow
      // direct access to them within the .htaccess file
      $not_handled_files = array();
      foreach ($this->list as $elt)
        if ($elt[$type_of_file_key] === FALSE) // File is not handled, we need to allow direct access
          array_push($not_handled_files, $elt['filename']);

      if (count($not_handled_files) > 0) {
        fwrite($hdl, '<FilesMatch "(' .
          ($type_of_file == 'T' ? $conf['prefix_thumbnail'] : '') .
          implode('|'.($type_of_file == 'T' ? $conf['prefix_thumbnail'] : ''), $not_handled_files) .
          ')">' .
            "\nallow from all\n</FilesMatch>\n\n"
          );
      }
    }

    fwrite($hdl, "deny from all\n");
    fwrite($hdl, current($htaccess_extra_content));
    fclose($hdl);

    $this->logInfo($file, 'successfully created');
  }

  /**
   * logError - add an entry in to the error log
   *
   * @path: path related to the error (filename for instance)
   * @type: type of error
   */
  function logError($path, $type) {
    array_push(
               $this->return['errors'],
               array('path' => $path,
                     'type' => $type,
                    )
              );
  }

  /**
   * logInfo - add an entry in to the information log
   *
   * @path: path related to the information (filename for instance)
   * @info: extra string providing more details
   */
  function logInfo($path, $info) {
    array_push(
               $this->return['infos'],
               array('path' => $path,
                     'type' => $info,
                    )
              );
  }

} // End of class HtaccesManagement

function secureImages_htaccess_creation($type, $simulate = FALSE) {
  if (!defined('PHPWG_ROOT_PATH'))
  {
    die('Hacking attempt!');
  }
  if ($type != 'selective' && $type != 'complete') {
    die('Invalid .htaccess type requested. Really not normal, quitting!');
  }

  $return = array('infos' => array(), 'errors' => array());
  if ($simulate)
    array_push(
        $return['infos'],
        array('path' => '',
          'info' => '!! SIMULATION MODE, NO MODIFICATION MADE TO FILESYSTEM NOR TYPE OF .HTACCESS WITHIN THE CONFIGURATION OPTIONS !!'
          )
        );

  /**
   * What kind of .htaccess type must we create ?
   */
  if ($type == 'complete') {
    // We only need to clean all previous .htaccess files and create a single one
    // in the root directory
    secureImages_htaccess_removal($simulate);
    $galleries_dir = PHPWG_ROOT_PATH . 'galleries';
    $not_handled_files = new HtaccesManagement(0, '', $simulate, &$return);
    $not_handled_files->writeRootHtaccess();
    return $return;
  }

  // Checking if we are working only on user-selected categories
  $opts['category_id'] = '';
  $opts['recursive'] = true;
  if (isset($_POST['cat']))
  {
    $opts['category_id'] = $_POST['cat'];
    // recursive ?
    if (!isset($_POST['subcats-included']) or $_POST['subcats-included'] != 1)
    {
      $opts['recursive'] = false;
    }
  }

  if (!isset($site_id))
    $site_id = 1;

  // Getting list of categories' ids
  $cat_ids = get_categories_ids($site_id, $opts);
  $cat_fulldirs = get_fulldirs($cat_ids);

  $query = '
    SELECT images.id AS img_id, path, storage_category_id, GROUP_CONCAT(DISTINCT status) AS status
      FROM '.IMAGES_TABLE.' AS images
        JOIN '.IMAGE_CATEGORY_TABLE.' AS image_category
          ON images.id = image_category.image_id 
        JOIN '.CATEGORIES_TABLE.' AS categories
          ON image_category.category_id = categories.id 
      WHERE images.storage_category_id IN ('.implode(',', $cat_ids).')' .
      ( isset($_POST['file_types']) ? // Do not include files whose extension is 
          ' AND path NOT LIKE "%.' .  // in the exclude (aka "Do not protect") list
          implode( '" AND path NOT LIKE "%.', array_map("mysql_escape_string", $_POST['file_types'])) . '"' 
        : 
          ''
      ) . '
      GROUP BY images.id
      ORDER BY storage_category_id ASC;';

  $result = pwg_query($query);

  unset($last_physical_category);

  $cat_ids_written_htaccess = array();

  while ($row = mysql_fetch_array($result)) {
    if (!isset($last_physical_category) || $last_physical_category !== $row['storage_category_id']) {
      // We are in another category (or we just started)
      if (isset($file)) {
        $not_handled_files->write();
      }

      $file = preg_replace('|(\.\/)+|',
                          './',
                          PHPWG_ROOT_PATH .
                            dirname($row['path']) .
                            '/.htaccess'
                         );

      array_push($cat_ids_written_htaccess, $row['storage_category_id']);

      $not_handled_files = new HtaccesManagement($row['storage_category_id'], dirname($file), $simulate, &$return);
      $last_physical_category = $row['storage_category_id'];
    }

    // Checking for public files that we will therefore not handle
    if (strstr($row['status'], 'public') !== FALSE) {
      if (!isset($not_handled_files))
        $not_handled_files = new HtaccesManagement($row['storage_category_id'], dirname($file), $simulate, &$return);
      $not_handled_files->add($row['img_id'], basename($row['path']));
    }
  }
  if (isset($file) && isset($not_handled_files)) {
    $not_handled_files->write();
  }
  
  writeAllDefaultHtAccessFiles($cat_fulldirs, $cat_ids, $cat_ids_written_htaccess, $simulate, $return);

  return $return;
}

function secureImages_htaccess_removal($simulate = FALSE) {
  if (!defined('PHPWG_ROOT_PATH'))
  {
    die('Hacking attempt!');
  }

  $opts['category_id'] = '';
  $opts['recursive'] = true;

  if (isset($_POST['cat']))
  {
    $opts['category_id'] = $_POST['cat'];
    // recursive ?
    if (!isset($_POST['subcats-included']) or $_POST['subcats-included'] != 1)
    {
      $opts['recursive'] = false;
    }
  }
  //  $start = get_moment();
  if (!isset($site_id))
    $site_id = 1;

  $cat_ids = get_categories_ids($site_id, $opts);
  // Getting directories of categories
  $dirs = get_fulldirs($cat_ids);
  if (count($dirs) == 0)
    break;

  $return = array('infos' => array(), 'errors' => array());

  if ($simulate)
    array_push(
        $return['infos'],
        array('path' => '',
          'info' => '!! Simulation mode, no modification made to filesystem nor type of .htaccess within the configuration options !!'
          )
        );

  foreach (array_values($dirs) as $dirs_elt) {
    removeAllHtaccessFiles($dirs_elt, $simulate, $return);
  }
  // Removing also the root .htacces within /galleries
  $temp_file = PHPWG_ROOT_PATH . '/galleries/.htaccess';
  removeHtaccessFile($temp_file, $simulate, $return);

  return $return;
}

function writeAllDefaultHtAccessFiles(&$dirs, &$all_ids, &$exclude_ids, $simulate, &$return) {
  foreach ($all_ids as $all_ids_elt) {
    if (in_array($all_ids_elt, $exclude_ids))
      break;
    $not_handled_files = new HtaccesManagement($all_ids_elt, $dirs[$all_ids_elt], $simulate, &$return);
    $not_handled_files->write();
  }
}

/** REMOVE **
function writeAllHtaccessFiles( &$dir, &$not_handled_files, $simulate, &$return ) {
  $temp_dir = $dir.'/.htaccess';
  writeHtaccessFile($temp_dir, $not_handled_files, $simulate, $return, FALSE);
  $temp_dir = $dir.'/thumbnail/.htaccess';
  writeHtaccessFile($temp_dir, $not_handled_files, $simulate, $return, TRUE);
  if (file_exists($dir.'/pwg_high')) {
    $temp_dir = $dir.'/pwg_high/.htaccess';
    writeHtaccessFile($temp_dir, $not_handled_files, $simulate, $return, FALSE);
  }
}

function writeHtaccessFile( &$file, &$not_handled_files, $simulate, &$return, $isThumb ) {
  global $conf;
  global $cfg_obj;

  if (!is_writable_extended($file)) {
    array_push(
               $return['errors'],
               array('path' => $file,
                     'type' => 'SECUREIMAGES-HTACCESS-NOT-WRITEABLE'
                    )
              );
    return;
  }
  elseif ($simulate) {
    array_push(
               $return['infos'],
               array('path' => $file,
                     'info' => 'successfully created'
                    )
              );
    return;
  }

  $hdl = @fopen( $file, 'w' );
  
  if (!$hdl) {
    array_push(
               $return['errors'],
               array('path' => $file,
                     'type' => 'SECUREIMAGES-UNKNOWN'
                    )
              );
    return;
  }

  // Do we have extra content for our generated .htaccess files ?
  $htaccess_extra_content = explode('__WRITE_HERE__', $cfg_obj->get('htaccess_extra_content'));
  if (count($htaccess_extra_content) > 1) {
    fwrite($hdl, current($htaccess_extra_content) . "\n");
    next($htaccess_extra_content);
  }

  if ($_POST['htaccess_type'] == 'selective') {
    if (isset($_POST['file_types']) && count($_POST['file_types']) > 0) {
      fwrite($hdl, '<FilesMatch "\.(' . 
        implode('|', $_POST['file_types']) . 
        ')$>' .
        "\nallow from all\n</FilesMatch>\n\n"
        );
    }

    if (count($not_handled_files) > 0) {
      fwrite($hdl, '<FilesMatch "(' .
        ($isThumb === TRUE ? $conf['prefix_thumbnail'] : '') .
        implode('|'.($isThumb === TRUE ? $conf['prefix_thumbnail'] : ''), $not_handled_files) .
        ')">' .
          "\nallow from all\n</FilesMatch>\n\n"
        );
    }
  }

  fwrite($hdl, "deny from all\n");
  fwrite($hdl, current($htaccess_extra_content));
  fclose($hdl);

  array_push(
             $return['infos'],
             array('path' => $file,
                   'info' => 'successfully created'
                  )
            );
}
** END REMOVE **/

function removeAllHtaccessFiles( &$dir, $simulate, &$return ) {
  $temp_dir = $dir.'/.htaccess';
  removeHtaccessFile($temp_dir, $simulate, $return);
  $temp_dir = $dir.'/thumbnail/.htaccess';
  removeHtaccessFile($temp_dir, $simulate, $return);
  if (file_exists($dir.'/pwg_high')) {
    $temp_dir = $dir.'/pwg_high/.htaccess';
    removeHtaccessFile($temp_dir, $simulate, $return);
  }
}

function removeHtaccessFile( &$file, $simulate, &$return ) {
  if (file_exists($file)) {
    if (!$simulate) { 
      unlink($file) ?  
        array_push(
          $return['infos'],
          array('path' => $file,
                'info' => 'successfully removed'
          )
        )
      : array_push(
          $return['errors'],
          array('path' => $file,
                'type' => 'SECUREIMAGES-HTACCESS-NOT-REMOVABLE'
          )
        );
    }
    else {
      is_writable_extended($file) ?  
        array_push(
          $return['infos'],
          array('path' => $file,
                'info' => 'successfully removed'
          )
        )
      : array_push(
          $return['errors'],
          array('path' => $file,
                'type' => 'SECUREIMAGES-HTACCESS-NOT-REMOVABLE'
          )
        );
    }
  }
}

function is_writable_extended(&$file) {
  if ( (!file_exists($file) and is_writable(dirname($file))) or
       is_writable($file)
     )
    return true;
}

function get_categories_ids($site_id, &$opts) {
  $cat_ids = array();

  $query = '
    SELECT id
      FROM '.CATEGORIES_TABLE.'
      WHERE site_id = '.$site_id.'
      AND dir IS NOT NULL';
  if (is_numeric($opts['category_id']))
  {
    if ($opts['recursive'])
    {
      $query.= '
        AND uppercats REGEXP \'(^|,)'.$opts['category_id'].'(,|$)\'
        ';
    }
    else
    {
      $query.= '
        AND id = '.$opts['category_id'].'
        ';
    }
  }
  $query.= '
    ;';
  $result = pwg_query($query);
  while ($row = mysql_fetch_array($result))
  {
    array_push($cat_ids, $row['id']);
  }

  if (count($cat_ids) == 0)
  {
    break;
  }

  return $cat_ids;
}

?>
